@@ -60,12 +60,10 @@ namespace Banshee.Collection.Database
 
     public class DatabaseTrackInfo : TrackInfo
     {
-        private static DatabaseTrackModelProvider<DatabaseTrackInfo> provider = new DatabaseTrackModelProvider<DatabaseTrackInfo> (
-            ServiceManager.DbConnection
-        );
+        private static DatabaseTrackModelProvider<DatabaseTrackInfo> provider;
 
         public static DatabaseTrackModelProvider<DatabaseTrackInfo> Provider {
-            get { return provider; }
+            get { return provider ?? provider = new DatabaseTrackModelProvider<DatabaseTrackInfo> (ServiceManager.DbConnection); }
         }
 
         private bool? artist_changed = null, album_changed = null;
@@ -122,7 +120,7 @@ namespace Banshee.Collection.Database
         }
 
         public DatabaseAlbumInfo Album {
-            get { return DatabaseAlbumInfo.FindOrCreate (DatabaseArtistInfo.FindOrCreate (AlbumArtist), AlbumTitle); }
+            get { return DatabaseAlbumInfo.FindOrCreate (DatabaseArtistInfo.FindOrCreate (AlbumArtist), AlbumTitle, IsCompilation); }
         }
 
         private static bool notify_saved = true;
@@ -224,8 +222,7 @@ namespace Banshee.Collection.Database
                 if (value == null)
                     return;
 
-                // Overwrite the AlbumArtist if it was set to our old value
-                if (String.IsNullOrEmpty (AlbumArtist) || AlbumArtist == ArtistName) {
+                if (!IsCompilation) {
                     AlbumArtist = value;
                 }
 
@@ -260,6 +257,15 @@ namespace Banshee.Collection.Database
             }
         }
         
+        [VirtualDatabaseColumn ("IsCompilation", "CoreAlbums", "AlbumID", "AlbumID")]
+        public override bool IsCompilation {
+            get { return base.IsCompilation; }
+            set {
+                base.IsCompilation = value;
+                album_changed = album_changed != null;
+            }
+        }
+        
         private static string CleanseString (string input, string old_val)
         {
             if (input == old_val)
@@ -267,10 +273,7 @@ namespace Banshee.Collection.Database
                     
             if (input != null)
                 input = input.Trim ();
-                
-            if (input == String.Empty)
-                return null;
-                
+            
             if (input == old_val)
                 return null;
             
@@ -343,6 +346,18 @@ namespace Banshee.Collection.Database
             get { return base.FileSize; }
             set { base.FileSize = value; }
         }
+
+        [DatabaseColumn]
+        public override long FileModifiedStamp {
+            get { return base.FileModifiedStamp; }
+            set { base.FileModifiedStamp = value; }
+        }
+
+        [DatabaseColumn]
+        public override DateTime LastSyncedStamp {
+            get { return base.LastSyncedStamp; }
+            set { base.LastSyncedStamp = value; }
+        }
         
         [DatabaseColumn ("Attributes")]
         public override TrackMediaAttributes MediaAttributes {
@@ -360,6 +375,11 @@ namespace Banshee.Collection.Database
         protected string TitleLowered {
             get { return TrackTitle == null ? null : TrackTitle.ToLower (); }
         }
+
+        [DatabaseColumn(Select = false)]
+        public override string MetadataHash {
+            get { return base.MetadataHash; }
+        }
         
         [DatabaseColumn]
         public override int TrackNumber {
@@ -373,10 +393,16 @@ namespace Banshee.Collection.Database
             set { base.TrackCount = value; }
         }
         
+        [DatabaseColumn ("Disc")]
+        public override int DiscNumber {
+            get { return base.DiscNumber; }
+            set { base.DiscNumber = value; }
+        }
+
         [DatabaseColumn]
-        public override int Disc {
-            get { return base.Disc; }
-            set { base.Disc = value; }
+        public override int DiscCount {
+            get { return base.DiscCount; }
+            set { base.DiscCount = value; }
         }
         
         [DatabaseColumn]
@@ -403,6 +429,18 @@ namespace Banshee.Collection.Database
             set { base.Composer = value; }
         }
 
+        [DatabaseColumn]
+        public override string Conductor {
+            get { return base.Conductor; }
+            set { base.Conductor = value; }
+        }
+
+        [DatabaseColumn]
+        public override string Grouping {
+            get { return base.Grouping; }
+            set { base.Grouping = value; }
+        }
+
         [DatabaseColumn]
         public override string Copyright {
             get { return base.Copyright; }
@@ -421,7 +459,19 @@ namespace Banshee.Collection.Database
             set { base.Comment = value; }
         }
         
-        [DatabaseColumn ("Rating")]
+        [DatabaseColumn("BPM")]
+        public override int Bpm {
+            get { return base.Bpm; }
+            set { base.Bpm = value; }
+        }
+
+        [DatabaseColumn]
+        public override int BitRate {
+            get { return base.BitRate; }
+            set { base.BitRate = value; }
+        }
+        
+        [DatabaseColumn("Rating")]
         protected int rating;
         public override int Rating {
             get { return rating; }
@@ -492,7 +542,6 @@ namespace Banshee.Collection.Database
                 }
                 
                 playback_error = value; 
-                Save ();
             }
         }
 
