@@ -107,26 +107,31 @@ namespace Banshee.Playlist
 
 #region Constructors
 
-        public PlaylistSource (string name, int primarySourceId) : this (name, null, primarySourceId)
+        public PlaylistSource (string name, PrimarySource parent) : base (generic_name, name, parent)
+        {
+            SetProperties ();
+        }
+
+        protected PlaylistSource (string name, int dbid, PrimarySource parent) : this (name, dbid, -1, 0, parent, 0, false)
         {
         }
 
-        protected PlaylistSource (string name, int? dbid, int primarySourceId) : this (name, dbid, -1, 0, primarySourceId, 0)
+        protected PlaylistSource (string name, int dbid, int sortColumn, int sortType, PrimarySource parent, int count, bool is_temp)
+            : base (generic_name, name, dbid, sortColumn, sortType, parent, is_temp)
         {
-        }
-
-        protected PlaylistSource (string name, int? dbid, int sortColumn, int sortType, int primarySourceId, int count)
-            : base (generic_name, name, dbid, sortColumn, sortType, primarySourceId)
-        {
-            Properties.SetString ("Icon.Name", "source-playlist");
-            Properties.SetString ("RemoveTracksActionLabel", Catalog.GetString ("Remove From Playlist"));
-            Properties.SetString ("UnmapSourceActionLabel", Catalog.GetString ("Delete Playlist"));
-            DbId = dbid;
+            SetProperties ();
             SavedCount = count;
         }
 
 #endregion
 
+        private void SetProperties ()
+        {
+            Properties.SetString ("Icon.Name", "source-playlist");
+            Properties.SetString ("RemoveTracksActionLabel", Catalog.GetString ("Remove From Playlist"));
+            Properties.SetString ("UnmapSourceActionLabel", Catalog.GetString ("Delete Playlist"));
+        }
+
 #region Source Overrides
 
         public override bool AcceptsInputFromSource (Source source)
@@ -145,6 +150,17 @@ namespace Banshee.Playlist
 
 #region AbstractPlaylist overrides
 
+        protected override void AfterInitialized ()
+        {
+            base.AfterInitialized ();
+            if (PrimarySource != null) {
+                PrimarySource.TracksChanged += HandleTracksChanged;
+                PrimarySource.TracksDeleted += HandleTracksDeleted;
+            }
+
+            TrackModel.CanReorder = true;
+        }
+
         protected override void Update ()
         {
             ServiceManager.DbConnection.Execute (new HyenaSqliteCommand (
@@ -153,19 +169,20 @@ namespace Banshee.Playlist
                         SET Name = ?,
                             SortColumn = ?,
                             SortType = ?,
-                            CachedCount = ?
+                            CachedCount = ?,
+                            IsTemporary = ?
                         WHERE PlaylistID = ?",
                     SourceTable
-                ), Name, -1, 0, Count, dbid
+                ), Name, -1, 0, Count, IsTemporary, dbid
             ));
         }
 
         protected override void Create ()
         {
             DbId = ServiceManager.DbConnection.Execute (new HyenaSqliteCommand (
-                @"INSERT INTO CorePlaylists (PlaylistID, Name, SortColumn, SortType, PrimarySourceID)
-                    VALUES (NULL, ?, ?, ?, ?)",
-                Name, -1, 1, PrimarySourceId //SortColumn, SortType
+                @"INSERT INTO CorePlaylists (PlaylistID, Name, SortColumn, SortType, PrimarySourceID, IsTemporary)
+                    VALUES (NULL, ?, ?, ?, ?, ?)",
+                Name, -1, 1, PrimarySourceId, IsTemporary //SortColumn, SortType
             ));
         }
 
@@ -173,14 +190,6 @@ namespace Banshee.Playlist
 
 #region DatabaseSource overrides
 
-        // We can delete tracks only if our parent can
-        public override bool CanDeleteTracks {
-            get {
-                DatabaseSource ds = Parent as DatabaseSource;
-                return ds != null && ds.CanDeleteTracks;
-            }
-        }
-
         // We can add tracks only if our parent can
         public override bool CanAddTracks {
             get {
@@ -189,6 +198,15 @@ namespace Banshee.Playlist
             }
         }
 
+        // We can remove tracks only if our parent can
+        public override bool CanRemoveTracks {
+            get {
+                return (Parent is PrimarySource)
+                    ? !(Parent as PrimarySource).PlaylistsReadOnly
+                    : true;
+            }
+        }
+
 #endregion
 
 #region IUnmapableSource Implementation
@@ -205,18 +223,10 @@ namespace Banshee.Playlist
                 ));
             }
 
-            Remove ();
+            ThreadAssist.ProxyToMain (Remove);
             return true;
         }
 
-        public virtual bool CanUnmap {
-            get { return true; }
-        }
-
-        public virtual bool ConfirmBeforeUnmap {
-            get { return true; }
-        }
-
 #endregion
 
         protected void AddTrack (int track_id)
@@ -328,29 +338,33 @@ namespace Banshee.Playlist
             }
         }
 
-        public static IEnumerable<PlaylistSource> LoadAll (int primary_id)
+        public static IEnumerable<PlaylistSource> LoadAll (PrimarySource parent)
         {
-            using (IDataReader reader = ServiceManager.DbConnection.Query (
-                @"SELECT PlaylistID, Name, SortColumn, SortType, PrimarySourceID, CachedCount FROM CorePlaylists 
-                    WHERE Special = 0 AND PrimarySourceID = ?", primary_id)) {
+            ClearTemporary ();
+            using (HyenaDataReader reader = new HyenaDataReader (ServiceManager.DbConnection.Query (
+                @"SELECT PlaylistID, Name, SortColumn, SortType, PrimarySourceID, CachedCount, IsTemporary FROM CorePlaylists 
+                    WHERE Special = 0 AND PrimarySourceID = ?", parent.DbId))) {
                 while (reader.Read ()) {
                     yield return new PlaylistSource (
-                        reader[1] as string, Convert.ToInt32 (reader[0]),
-                        Convert.ToInt32 (reader[2]), Convert.ToInt32 (reader[3]), Convert.ToInt32 (reader[4]),
-                        Convert.ToInt32 (reader[5])
+                        reader.Get<string> (1), reader.Get<int> (0),
+                        reader.Get<int> (2), reader.Get<int> (3), parent,
+                        reader.Get<int> (5), reader.Get<bool> (6)
                     );
                 }
             }
         }
 
-        public override void SetParentSource (Source parent)
+        private static bool temps_cleared = false;
+        private static void ClearTemporary ()
         {
-            base.SetParentSource (parent);
-
-            PrimarySource primary = parent as PrimarySource;
-            if (primary != null) {
-                primary.TracksChanged += HandleTracksChanged;
-                primary.TracksDeleted += HandleTracksDeleted;
+            if (!temps_cleared) {
+                temps_cleared = true;
+                ServiceManager.DbConnection.Execute (@"
+                    BEGIN TRANSACTION;
+                        DELETE FROM CorePlaylistEntries WHERE PlaylistID IN (SELECT PlaylistID FROM CorePlaylists WHERE IsTemporary = 1);
+                        DELETE FROM CorePlaylists WHERE IsTemporary = 1;
+                    COMMIT TRANSACTION"
+                );
             }
         }
         
