@@ -69,9 +69,16 @@ namespace Banshee.Sources
         }
 
         private RateLimiter reload_limiter;
-        
-        public DatabaseSource (string generic_name, string name, string id, int order) : base (generic_name, name, order, id)
+
+        public DatabaseSource (string generic_name, string name, string id, int order) : this (generic_name, name, id, order, null)
         {
+        }
+        
+        public DatabaseSource (string generic_name, string name, string id, int order, Source parent) : base (generic_name, name, order, id)
+        {
+            if (parent != null) {
+                SetParentSource (parent);
+            }
             DatabaseSourceInitialize ();
         }
 
@@ -269,6 +276,14 @@ namespace Banshee.Sources
         {
             return CanAddTracks && source != this;
         }
+                
+        public override bool HasViewableTrackProperties {
+            get { return true; }
+        }
+        
+        public override bool HasEditableTrackProperties {
+            get { return true; }
+        }
 
 #endregion
         
@@ -577,9 +592,9 @@ namespace Banshee.Sources
         
         public override void Deactivate ()
         {
-            DatabaseTrackModel.InvalidateCache ();
+            DatabaseTrackModel.InvalidateCache (false);
             foreach (IFilterListModel filter in AvailableFilters) {
-                filter.InvalidateCache ();
+                filter.InvalidateCache (false);
             }
         }
 
@@ -638,10 +653,10 @@ namespace Banshee.Sources
 
         protected void InvalidateCaches ()
         {
-            track_model.InvalidateCache ();
+            track_model.InvalidateCache (true);
 
             foreach (IFilterListModel filter in CurrentFilters) {
-                filter.InvalidateCache ();
+                filter.InvalidateCache (true);
             }
         }
 
