@@ -33,10 +33,17 @@ using Mono.Unix;
 using Mono.Addins;
 using Gtk;
 
+using Hyena.Gui;
+using Hyena.Widgets;
+
+using Banshee.Kernel;
+using Banshee.Sources;
+using Banshee.ServiceStack;
 using Banshee.Collection;
 using Banshee.Collection.Database;
-using Banshee.ServiceStack;
+using Banshee.Configuration.Schema;
 
+using Banshee.Widgets;
 using Banshee.Gui.Dialogs;
 using Banshee.Collection.Gui;
 
@@ -44,26 +51,6 @@ namespace Banshee.Gui.TrackEditor
 {
     public class TrackEditorDialog : BansheeDialog
     {
-        public static void RunEdit (TrackListModel model)
-        {
-            Run (model, EditorMode.Edit);
-        }
-        
-        public static void RunView (TrackListModel model)
-        {
-            Run (model, EditorMode.View);
-        }
-        
-        public static void Run (TrackListModel model, EditorMode mode)
-        {
-            TrackEditorDialog track_editor = new TrackEditorDialog (model, mode);
-            try {
-                track_editor.Run ();
-            } finally {
-                track_editor.Destroy ();
-            }
-        }
-    
         public delegate void EditorTrackOperationClosure (EditorTrackInfo track);
     
         private VBox main_vbox;
@@ -74,6 +61,20 @@ namespace Banshee.Gui.TrackEditor
         private Label header_artist_label;
         private Label header_album_label;
         private Label edit_notif_label;
+        private object tooltip_host;
+        
+        private DateTime first_change_time;
+        private bool changes_made;
+        public bool ChangesMade {
+            get { return changes_made; }
+            private set {
+                if (!changes_made && value) {
+                    first_change_time = DateTime.Now;
+                }
+                
+                changes_made = value;
+            }
+        }
         
         private Notebook notebook;
         public Notebook Notebook {
@@ -82,7 +83,7 @@ namespace Banshee.Gui.TrackEditor
     
         private Button nav_backward_button;
         private Button nav_forward_button;
-        private Button sync_all_button;
+        private PulsingButton sync_all_button;
         
         private EditorMode mode;
         
@@ -90,7 +91,8 @@ namespace Banshee.Gui.TrackEditor
         
         public event EventHandler Navigated;
         
-        public TrackEditorDialog (TrackListModel model, EditorMode mode) : base (Catalog.GetString ("Track Editor"))
+        private TrackEditorDialog (TrackListModel model, EditorMode mode) : base (
+            mode == EditorMode.Edit ? Catalog.GetString ("Track Editor") : Catalog.GetString ("Track Properties"))
         {
             this.mode = mode;
             
@@ -107,6 +109,8 @@ namespace Banshee.Gui.TrackEditor
                 SetSizeRequest (400, 500);
             }
             
+            tooltip_host = TooltipSetter.CreateHost ();
+            
             AddNavigationButtons ();
             
             main_vbox = new VBox ();
@@ -134,11 +138,13 @@ namespace Banshee.Gui.TrackEditor
             nav_backward_button.UseStock = true;
             nav_backward_button.Clicked += delegate { NavigateBackward (); };
             nav_backward_button.Show ();
+            TooltipSetter.Set (tooltip_host, nav_backward_button, Catalog.GetString ("Show the previous track"));
             
             nav_forward_button = new Button (Stock.GoForward);
             nav_forward_button.UseStock = true;
             nav_forward_button.Clicked += delegate { NavigateForward (); };
             nav_forward_button.Show ();
+            TooltipSetter.Set (tooltip_host, nav_forward_button, Catalog.GetString ("Show the next track"));
             
             ActionArea.PackStart (nav_backward_button, false, false, 0);
             ActionArea.PackStart (nav_forward_button, false, false, 0);
@@ -197,6 +203,7 @@ namespace Banshee.Gui.TrackEditor
                         page.Initialize (this);
                         page.Widget.Show ();
                     }
+                    page.Changed += OnPageChanged;
                 } catch (Exception e) {
                     Hyena.Log.Exception ("Invalid NotebookPage extension node. Should implement ITrackEditorPage.", e);
                 }
@@ -227,11 +234,39 @@ namespace Banshee.Gui.TrackEditor
             button_box.Spacing = 6;
                        
             if (TrackCount > 1) {
-                sync_all_button = new Button ();
+                sync_all_button = new PulsingButton ();
+                sync_all_button.FocusInEvent += delegate {
+                    ForeachWidget<SyncButton> (delegate (SyncButton button) {
+                        button.StartPulsing ();
+                    });
+                };
+                
+                sync_all_button.FocusOutEvent += delegate {
+                    if (sync_all_button.State == StateType.Prelight) {
+                        return;
+                    }
+                    
+                    ForeachWidget<SyncButton> (delegate (SyncButton button) {
+                        button.StopPulsing ();
+                    });
+                };
+                
+                sync_all_button.StateChanged += delegate {
+                    if (sync_all_button.HasFocus) {
+                        return;
+                    }
+                
+                    ForeachWidget<SyncButton> (delegate (SyncButton button) {
+                        if (sync_all_button.State == StateType.Prelight) {
+                            button.StartPulsing ();
+                        } else {
+                            button.StopPulsing ();
+                        }
+                    });
+                };
+                
                 sync_all_button.Clicked += delegate {
-                    for (int i = 0; i < notebook.NPages; i++) {
-                        InvokeFieldSync (notebook.GetNthPage (i) as Container);
-                    }        
+                    InvokeFieldSync ();
                 };
                 
                 Alignment alignment = new Alignment (0.5f, 0.5f, 0.0f, 0.0f);
@@ -242,6 +277,9 @@ namespace Banshee.Gui.TrackEditor
                 alignment.Add (box);
                 sync_all_button.Add (alignment);
                 
+                TooltipSetter.Set (tooltip_host, sync_all_button, Catalog.GetString (
+                    "Apply the values of all common fields set for this track to all of the tracks selected in this editor"));
+                    
                 button_box.PackStart (sync_all_button, false, false, 0);
                 
                 foreach (Widget child in ActionArea.Children) {
@@ -257,25 +295,41 @@ namespace Banshee.Gui.TrackEditor
             button_box.ShowAll ();
         }
         
-        private void InvokeFieldSync (Container container)
+        private delegate void WidgetAction<T> (T widget) where T : class;
+        
+        private void ForeachWidget<T> (WidgetAction<T> action) where T : class
+        {
+            for (int i = 0; i < notebook.NPages; i++) {
+                ForeachWidget (notebook.GetNthPage (i) as Container, action);
+            }     
+        }
+        
+        private void ForeachWidget<T> (Container container, WidgetAction<T> action) where T : class
         {
             if (container == null) {
                 return;
             }
             
             foreach (Widget child in container.Children) {
-                SyncButton sync = child as SyncButton;
-                if (sync != null) {
-                    sync.Click ();
+                T widget = child as T;
+                if (widget != null) {
+                    action (widget);
                 } else {
                     Container child_container = child as Container;
                     if (child_container != null) {
-                        InvokeFieldSync (child_container);
+                        ForeachWidget<T> (child_container, action);
                     }
                 }
             }
         }
         
+        private void InvokeFieldSync ()
+        {
+            ForeachWidget<SyncButton> (delegate (SyncButton button) {
+                button.Click ();
+            });
+        }
+        
         private int action_area_children_allocated = 0;
         
         private void OnActionAreaChildSizeAllocated (object o, SizeAllocatedArgs args)
@@ -288,6 +342,11 @@ namespace Banshee.Gui.TrackEditor
             sync_all_button.WidthRequest = (children[1].Allocation.X + 
                 children[1].Allocation.Width) - children[0].Allocation.X - 1;
         }
+        
+        private void OnPageChanged (object o, EventArgs args)
+        {
+            ChangesMade = true;
+        }
 
 #endregion
         
@@ -340,10 +399,20 @@ namespace Banshee.Gui.TrackEditor
                 header_image_frame.ShadowType = ShadowType.In;
             }
             
+            // Disconnect all the undo adapters
+            ForeachWidget<ICanUndo> (delegate (ICanUndo undoable) {
+                undoable.DisconnectUndo ();
+            });
+            
             foreach (ITrackEditorPage page in pages) {
                 page.LoadTrack (editor_track);
             }
             
+            // Connect all the undo adapters
+            ForeachWidget<ICanUndo> (delegate (ICanUndo undoable) {
+                undoable.ConnectUndo (editor_track);
+            });
+            
             // Update Navigation
             if (TrackCount > 0 && nav_backward_button != null && nav_forward_button != null) {
                 nav_backward_button.Sensitive = CanGoBackward;
@@ -366,12 +435,22 @@ namespace Banshee.Gui.TrackEditor
         }
         
         public EditorTrackInfo LoadTrack (int index)
+        {
+            return LoadTrack (index, true);
+        }
+        
+        public EditorTrackInfo LoadTrack (int index, bool alwaysLoad)
         {
             TrackInfo source_track;
-            return LoadTrack (index, out source_track);
+            return LoadTrack (index, alwaysLoad, out source_track);
         }
         
         private EditorTrackInfo LoadTrack (int index, out TrackInfo sourceTrack)
+        {
+            return LoadTrack (index, true, out sourceTrack);
+        }
+        
+        private EditorTrackInfo LoadTrack (int index, bool alwaysLoad, out TrackInfo sourceTrack)
         {
             sourceTrack = GetTrack (index);
             EditorTrackInfo editor_track = null;
@@ -381,7 +460,7 @@ namespace Banshee.Gui.TrackEditor
                 return null;
             }
             
-            if (!edit_map.TryGetValue (sourceTrack, out editor_track)) {
+            if (!edit_map.TryGetValue (sourceTrack, out editor_track) && alwaysLoad) {
                 editor_track = new EditorTrackInfo (sourceTrack);
                 editor_track.EditorIndex = index;
                 editor_track.EditorCount = TrackCount;
@@ -438,6 +517,174 @@ namespace Banshee.Gui.TrackEditor
             get { return current_track_index >= 0 && current_track_index < TrackCount - 1; }
         }
 
+#endregion
+
+#region Saving
+
+        public void Save ()
+        {
+            List<int> primary_sources = new List<int> ();
+            
+            // TODO: wrap in db transaction
+            try {
+                DatabaseTrackInfo.NotifySaved = false;
+
+                for (int i = 0; i < TrackCount; i++) {
+                    // Save any tracks that were actually loaded into the editor
+                    EditorTrackInfo track = LoadTrack (i, false);
+                    if (track == null || track.SourceTrack == null) {
+                        continue;
+                    }
+                    
+                    SaveTrack (track);
+                    
+                    if (track.SourceTrack is DatabaseTrackInfo) {
+                        // If the source track is from the database, save its parent for notification later
+                        int id = (track.SourceTrack as DatabaseTrackInfo).PrimarySourceId;
+                        if (!primary_sources.Contains (id)) {
+                            primary_sources.Add (id);
+                        }
+                    }
+                }
+
+                // Finally, notify the affected primary sources
+                foreach (int id in primary_sources) {
+                    PrimarySource.GetById (id).NotifyTracksChanged ();
+                }
+            } finally {
+                DatabaseTrackInfo.NotifySaved = true;
+            }
+        }
+        
+        private void SaveTrack (EditorTrackInfo track)
+        {
+            TrackInfo.ExportableMerge (track, track.SourceTrack);
+            track.SourceTrack.Save ();
+                
+            if (LibrarySchema.WriteMetadata.Get ()) {
+                SaveToFile (track.SourceTrack);
+            }
+
+            if (LibrarySchema.MoveOnInfoSave.Get ()) {
+                MoveSavedFile (track.SourceTrack);
+            }
+
+            if (track.SourceTrack == ServiceManager.PlayerEngine.CurrentTrack) {
+                ServiceManager.PlayerEngine.TrackInfoUpdated ();
+            }
+        }
+        
+        private void SaveToFile (TrackInfo track)
+        {
+            Scheduler.Schedule (new Banshee.Streaming.SaveTrackMetadataJob (track), JobPriority.Highest);
+        }
+
+        private void MoveSavedFile (TrackInfo track)
+        {
+            Scheduler.Schedule (new MoveOnInfoSaveJob (track), JobPriority.Highest);
+        }
+
+#endregion
+
+#region Static Helpers
+
+        public static void RunEdit (TrackListModel model)
+        {
+            Run (model, EditorMode.Edit);
+        }
+        
+        public static void RunView (TrackListModel model)
+        {
+            Run (model, EditorMode.View);
+        }
+        
+        public static void Run (TrackListModel model, EditorMode mode)
+        {
+            TrackEditorDialog track_editor = new TrackEditorDialog (model, mode);
+            track_editor.Response += delegate (object o, ResponseArgs args) {
+                if (args.ResponseId == ResponseType.Ok) {
+                    track_editor.Save ();
+                } else if (track_editor.ChangesMade) {
+                    int changed_count = 0;
+                    for (int i = 0; i < track_editor.TrackCount; i++) {
+                        EditorTrackInfo track = track_editor.LoadTrack (i, false);
+                        if (track != null && track.Changed) {
+                            changed_count++;
+                        }
+                    }
+                    
+                    if (changed_count == 0) {
+                        // Shouldn't ever reach this
+                        track_editor.Destroy ();
+                        return;
+                    }
+                
+                    HigMessageDialog message_dialog = new HigMessageDialog (
+                        track_editor, DialogFlags.Modal, MessageType.Warning, ButtonsType.None, 
+                        
+                        String.Format (Catalog.GetPluralString (
+                            "Save the changes made to the open track?",
+                            "Save the changes made to {0} of {1} open tracks?",
+                            track_editor.TrackCount), changed_count, track_editor.TrackCount),
+                            
+                        String.Empty
+                    );
+                    
+                    UpdateCancelMessage (track_editor, message_dialog);
+                    uint timeout = 0;
+                    timeout = GLib.Timeout.Add (1000, delegate {
+                        bool result = UpdateCancelMessage (track_editor, message_dialog);
+                        if (!result) {
+                            timeout = 0;
+                        }
+                        return result;
+                    });
+                    
+                    message_dialog.AddButton (Catalog.GetString ("Close _without Saving"), ResponseType.Close, false);
+                    message_dialog.AddButton (Stock.Cancel, ResponseType.Cancel, false);
+                    message_dialog.AddButton (Stock.Save, ResponseType.Ok, true);
+                    
+                    try {
+                        switch ((ResponseType)message_dialog.Run ()) {
+                            case ResponseType.Ok:
+                                track_editor.Save ();
+                                break;
+                            case ResponseType.Close:
+                                break;
+                            case ResponseType.Cancel:
+                            case ResponseType.DeleteEvent:
+                                return;
+                        }
+                    } finally {
+                        if (timeout > 0) {
+                            GLib.Source.Remove (timeout);
+                        }
+                        message_dialog.Destroy ();
+                    }
+                }
+                
+                track_editor.Destroy ();
+            };
+            
+            track_editor.Run ();
+        }
+        
+        private static bool UpdateCancelMessage (TrackEditorDialog trackEditor, HigMessageDialog messageDialog)
+        {
+            if (messageDialog == null) {
+                return false;
+            }
+            
+            messageDialog.MessageLabel.Text = String.Format (Catalog.GetString (
+                "If you don't save, changes from the last {0} will be permanently lost."),
+                Banshee.Sources.DurationStatusFormatters.ApproximateVerboseFormatter (
+                    DateTime.Now - trackEditor.first_change_time
+                )
+            );
+            
+            return messageDialog.IsMapped;
+        }
+    
 #endregion
 
     }
