@@ -110,6 +110,22 @@ namespace Banshee.Gui
                 }
             }
         }
+
+        public void RemoveActionGroup (ActionGroup group)
+        {
+            RemoveActionGroup (group.Name);
+        }
+        
+        public ActionGroup FindActionGroup (string actionGroupId)
+        {
+            foreach (ActionGroup group in action_groups.Values) {
+                if (group.Name == actionGroupId) {
+                    return group;
+                }
+            }
+            
+            return null;
+        }
         
         public Action FindAction (string actionId)
         {
@@ -122,15 +138,8 @@ namespace Banshee.Gui
             string group_name = parts[0];
             string action_name = parts[1];
             
-            foreach (ActionGroup group in action_groups.Values) {
-                if (group.Name != group_name) {
-                    continue;
-                }
-                
-                return group.GetAction (action_name);
-            }
-            
-            return null;
+            ActionGroup group = FindActionGroup (group_name);
+            return group == null ? null : group.GetAction (action_name);
         }
         
         public void PopulateToolbarPlaceholder (Toolbar toolbar, string path, Widget item)
@@ -156,6 +165,13 @@ namespace Banshee.Gui
         }
         
         private void OnActiveSourceChanged (SourceEventArgs args)
+        {
+            // FIXME: Can't use an anonymous delegate here because of compiler
+            // bug in Mono 1.2.6
+            Banshee.Base.ThreadAssist.ProxyToMain (OnActiveSourceChangedGui);
+        }
+
+        private void OnActiveSourceChangedGui ()
         {
             if (active_source_uiid > 0) {
                 ui_manager.RemoveUi (active_source_uiid);
@@ -177,14 +193,26 @@ namespace Banshee.Gui
                 AddActionGroup (active_source_actions);
             }
                 
-            Assembly assembly = Assembly.GetAssembly (active_source.GetType ());
-            string ui_file = active_source.Properties.GetString ("ActiveSourceUIResource");
-            
+            Assembly assembly = 
+                active_source.Properties.Get<Assembly> ("ActiveSourceUIResource.Assembly") ??
+                Assembly.GetAssembly (active_source.GetType ());
+
+            active_source_uiid = AddUiFromFile (active_source.Properties.Get<string> ("ActiveSourceUIResource"), assembly);
+        }
+
+        public uint AddUiFromFileInCurrentAssembly (string ui_file)
+        {
+            return AddUiFromFile (ui_file, Assembly.GetCallingAssembly ());
+        }
+        
+        public uint AddUiFromFile (string ui_file, Assembly assembly)
+        {
             if (ui_file != null) {
                 using (StreamReader reader = new StreamReader (assembly.GetManifestResourceStream (ui_file))) {
-                    active_source_uiid = ui_manager.AddUiFromString (reader.ReadToEnd ());
+                    return ui_manager.AddUiFromString (reader.ReadToEnd ());
                 }
             }
+            return 0;
         }
         
         public Action this[string actionId] {
