@@ -34,14 +34,17 @@ using Mono.Unix;
 
 using Hyena;
 using Hyena.Data.Sqlite;
+
 using Banshee.Base;
 using Banshee.ServiceStack;
 using Banshee.Sources;
 using Banshee.Collection;
+using Banshee.Playlist;
 using Banshee.Collection.Database;
 using Banshee.Hardware;
 using Banshee.MediaEngine;
 using Banshee.MediaProfiles;
+using Banshee.Preferences;
 
 using Banshee.Dap.Gui;
 
@@ -49,7 +52,9 @@ namespace Banshee.Dap
 {
     public abstract class DapSource : RemovableSource, IDisposable
     {
+        private DapSync sync;
         private DapInfoBar dap_info_bar;
+        private Page page;
         // private DapPropertiesDisplay dap_properties_display;
         
         private IDevice device;
@@ -72,6 +77,11 @@ namespace Banshee.Dap
         protected MediaGroupSource VideoGroupSource {
             get { return video_group_source; }
         }
+
+        private MediaGroupSource podcast_group_source;
+        protected MediaGroupSource PodcastGroupSource {
+            get { return podcast_group_source; }
+        }
         
         protected DapSource ()
         {
@@ -80,12 +90,12 @@ namespace Banshee.Dap
         public virtual void DeviceInitialize (IDevice device)
         {
             this.device = device;
-            type_unique_id = device.Uuid;
+            TypeUniqueId = device.Uuid;
         }
 
         public override void Dispose ()
         {
-            PurgeBuiltinSmartPlaylists ();
+            PurgeTemporaryPlaylists ();
             PurgeTracks ();
             
             if (dap_info_bar != null) {
@@ -100,7 +110,7 @@ namespace Banshee.Dap
             dap_properties_display = null;*/
         }
         
-        private void PurgeBuiltinSmartPlaylists ()
+        private void PurgeTemporaryPlaylists ()
         {
             ServiceManager.DbConnection.Execute (new HyenaSqliteCommand (@"
                 BEGIN TRANSACTION;
@@ -110,6 +120,15 @@ namespace Banshee.Dap
                 COMMIT TRANSACTION",
                 DbId, DbId
             ));
+
+            ServiceManager.DbConnection.Execute (new HyenaSqliteCommand (@"
+                BEGIN TRANSACTION;
+                    DELETE FROM CorePlaylistEntries WHERE PlaylistID IN
+                        (SELECT PlaylistID FROM CorePlaylists WHERE PrimarySourceID = ?);
+                    DELETE FROM CorePlaylists WHERE PrimarySourceID = ?;   
+                COMMIT TRANSACTION",
+                DbId, DbId
+            ));
         }
         
         internal void RaiseUpdated ()
@@ -117,11 +136,15 @@ namespace Banshee.Dap
             OnUpdated ();
         }
 
+        public virtual void SyncPlaylists ()
+        {
+        }
+
 #region Source
 
         protected override void Initialize ()
         {
-            PurgeBuiltinSmartPlaylists ();
+            PurgeTemporaryPlaylists ();
             
             base.Initialize ();
             
@@ -133,7 +156,10 @@ namespace Banshee.Dap
             });
             
             Properties.Set<bool> ("Nereid.SourceContents.HeaderVisible", false);
-            
+            Properties.Set<System.Reflection.Assembly> ("ActiveSourceUIResource.Assembly", System.Reflection.Assembly.GetExecutingAssembly ());
+            Properties.SetString ("ActiveSourceUIResource", "ActiveSourceUI.xml");
+
+            sync = new DapSync (this);
             dap_info_bar = new DapInfoBar (this);
             Properties.Set<Gtk.Widget> ("Nereid.SourceContents.FooterWidget", dap_info_bar);
             
@@ -157,29 +183,49 @@ namespace Banshee.Dap
                     : new string [] { "taglib/mp3" };
             }
             
-            music_group_source = new MusicGroupSource (this);
-            video_group_source = new VideoGroupSource (this);
+            AddChildSource (music_group_source = new MusicGroupSource (this));
+            AddChildSource (video_group_source = new VideoGroupSource (this));
+            AddChildSource (podcast_group_source = new PodcastGroupSource (this));
 
-            AddChildSource (music_group_source);
-            AddChildSource (video_group_source);
+            BuildPreferences ();
+
+            ThreadAssist.ProxyToMain (delegate {
+                Properties.Set<Banshee.Sources.Gui.ISourceContents> ("Nereid.SourceContents", new DapContent (this));
+            });
+        }
+
+        private void BuildPreferences ()
+        {
+            page = new Page ();
+            Section main_section = new Section ();
+            main_section.Order = -1;
+            
+            space_for_data = CreateSchema<long> ("space_for_data", 0, "How much space, in bytes, to reserve for data on the device.", "");
+            main_section.Add (space_for_data);
+            page.Add (main_section);
+
+            foreach (Section section in sync.PreferenceSections) {
+                page.Add (section);
+            }
         }
         
-        public override void AddChildSource (Source child)
+        /*public override void AddChildSource (Source child)
         {
             if (child is Banshee.Playlist.AbstractPlaylistSource && !(child is MediaGroupSource)) {
                 Log.Information ("Note: playlists added to digital audio players within Banshee are not yet saved to the device.", true);
             }
             
             base.AddChildSource (child);
+        }*/
+
+        // Force to zero so that count doesn't show up
+        public override int Count {
+            get { return 0; }
         }
         
         public override bool HasProperties {
             get { return true; }
         }
-        
-        public override bool CanActivate {
-            get { return false; }
-        }
 
         public override void SetStatus (string message, bool can_close, bool is_spinning, string icon_name)
         {
@@ -200,7 +246,7 @@ namespace Banshee.Dap
 #endregion
         
 #region Track Management/Syncing   
-
+ 
         public void LoadDeviceContents ()
         {
             ThreadPool.QueueUserWorkItem (ThreadedLoadDeviceContents);
@@ -212,18 +258,57 @@ namespace Banshee.Dap
                 PurgeTracks ();
                 SetStatus (String.Format (Catalog.GetString ("Loading {0}"), Name), false);
                 LoadFromDevice ();
-                OnTracksAdded ();
                 HideStatus ();
+
+                sync.DapLoaded ();
+                sync.CalculateSync ();
+                if (sync.AutoSync) {
+                    sync.Sync ();
+                }
             } catch (Exception e) {
                 Log.Exception (e);
             }
         }
 
+        public void RemovePlaylists ()
+        {
+            // First remove any playlists on the device
+            List<Source> children = new List<Source> (sync.Dap.Children);
+            foreach (Source child in children) {
+                if (child is AbstractPlaylistSource && !(child is MediaGroupSource)) {
+                    (child as IUnmapableSource).Unmap ();
+                }
+            }
+        }
+
         protected virtual void LoadFromDevice ()
         {
         }
+
+        protected override void Eject ()
+        {
+            if (!Sync.Enabled) {
+                // If sync isn't enabled, then make sure we've written saved our playlists
+                // Track transfers happen immediately, but playlists saves don't
+                SyncPlaylists ();
+            }
+        }
+
+        HyenaSqliteCommand track_on_dap_query = new HyenaSqliteCommand (
+            "SELECT TrackID FROM CoreTracks WHERE PrimarySourceID = ? AND MetadataHash = ? LIMIT 1");
+
+        private void AttemptToAddTrackToDevice (DatabaseTrackInfo track, SafeUri fromUri)
+        {
+            // Ensure there's enough space
+            if (BytesAvailable - Banshee.IO.File.GetSize (fromUri) >= 0) {
+                // Ensure it's not already on the device
+                if (ServiceManager.DbConnection.Query<int> (track_on_dap_query, DbId, track.MetadataHash) == 0) {
+                    AddTrackToDevice (track, fromUri);
+                }
+            }
+        }
         
-        protected abstract void AddTrackToDevice (DatabaseTrackInfo track, SafeUri fromUri);  
+        protected abstract void AddTrackToDevice (DatabaseTrackInfo track, SafeUri fromUri);
 
         protected bool TrackNeedsTranscoding (TrackInfo track)
         {
@@ -249,6 +334,11 @@ namespace Banshee.Dap
             dap_properties.Add (new DapProperty (key, val));
         }
 
+        protected void AddYesNoDapProperty (string key, bool val)
+        {
+            AddDapProperty (key, val ? Catalog.GetString ("Yes") : Catalog.GetString ("No"));
+        }
+
         public IEnumerable<DapProperty> DapProperties {
             get { return dap_properties; }
         }
@@ -256,7 +346,7 @@ namespace Banshee.Dap
         protected override void AddTrackAndIncrementCount (DatabaseTrackInfo track)
         {
             if (!TrackNeedsTranscoding (track)) {
-                AddTrackToDevice (track, track.Uri);
+                AttemptToAddTrackToDevice (track, track.Uri);
                 IncrementAddedTracks ();
                 return;
             }
@@ -267,13 +357,13 @@ namespace Banshee.Dap
                 string format = System.IO.Path.GetExtension (track.Uri.LocalPath);
                 format = String.IsNullOrEmpty (format) ? Catalog.GetString ("Unknown") : format.Substring (1);
                 throw new ApplicationException (String.Format (Catalog.GetString (
-                    "The {0} format is not supported by the device, and no converter was found to convert it."), format));
+                    "The {0} format is not supported by the device, and no converter was found to convert it"), format));
             }
 
             TranscoderService transcoder = ServiceManager.Get<TranscoderService> ();
             if (transcoder == null) {
                 throw new ApplicationException (Catalog.GetString (
-                    "File format conversion is not supported for this device."));
+                    "File format conversion support is not available"));
             }
             
             transcoder.Enqueue (track, PreferredConfiguration, OnTrackTranscoded, OnTrackTranscodeCancelled);
@@ -284,7 +374,7 @@ namespace Banshee.Dap
             AddTrackJob.Status = String.Format ("{0} - {1}", track.ArtistName, track.TrackTitle);
             
             try {
-                AddTrackToDevice ((DatabaseTrackInfo)track, outputUri);
+                AttemptToAddTrackToDevice ((DatabaseTrackInfo)track, outputUri);
             } catch (Exception e) {
                 Log.Exception (e);
             }
@@ -311,7 +401,7 @@ namespace Banshee.Dap
 
             if (!String.IsNullOrEmpty (vendor) && !String.IsNullOrEmpty (product)) {
                 return new string [] { 
-                    String.Format ("{0}-{1}", vendor, product).Replace (' ', '-').ToLower (), 
+                    String.Format ("multimedia-player-{0}-{1}", vendor, product).Replace (' ', '-').ToLower (), 
                     FallbackIcon
                 };
             } else {
@@ -330,7 +420,15 @@ namespace Banshee.Dap
         protected virtual IDeviceMediaCapabilities MediaCapabilities {
             get { return device.MediaCapabilities; }
         }
-        
+
+        public Page Preferences {
+            get { return page; }
+        }
+
+        public DapSync Sync {
+            get { return sync; }
+        }
+
         private ProfileConfiguration preferred_config;
         private ProfileConfiguration PreferredConfiguration {
             get {
@@ -361,7 +459,37 @@ namespace Banshee.Dap
         public long BytesMusic {
             get { return MusicGroupSource == null ? 0 : MusicGroupSource.BytesUsed; }
         }
-        
+                    
+        public long BytesData {
+            get { return BytesUsed - BytesVideo - BytesMusic; }
+        }
+                    
+        public long BytesReserved {
+            get { return space_for_data.Get (); }
+            set { space_for_data.Set (value); }
+        }
+                    
+        public override long BytesAvailable {
+            get { return BytesCapacity - BytesUsed - Math.Max (0, BytesReserved - BytesData); }
+        }
+
+        public override bool CanRemoveTracks {
+            get { return base.CanRemoveTracks && !Sync.Enabled; }
+        }
+
+        public override bool CanDeleteTracks {
+            get { return base.CanDeleteTracks && !Sync.Enabled; }
+        }
+
+        public override bool CanAddTracks {
+            get { return base.CanAddTracks && !Sync.Enabled; }
+        }
+
+        public override bool PlaylistsReadOnly {
+            get { return Sync.Enabled || IsReadOnly; }
+        }
+            
+        private Banshee.Configuration.SchemaEntry<long> space_for_data;
 #endregion
         
     }
