@@ -43,13 +43,13 @@ using Banshee.Gui;
 
 namespace Banshee.AudioCd
 {
-    public class AudioCdSource : Source, ITrackModelSource, IUnmapableSource, IDurationAggregator, IDisposable
+    public class AudioCdSource : Source, ITrackModelSource, IUnmapableSource, 
+        IDurationAggregator, IFileSizeAggregator, IDisposable
     {
         private AudioCdService service;
         private AudioCdDiscModel disc_model;
         private SourceMessage query_message;
-        
-        
+
         public AudioCdSource (AudioCdService service, AudioCdDiscModel discModel) 
             : base (Catalog.GetString ("Audio CD"), discModel.Title, 400)
         {
@@ -78,6 +78,10 @@ namespace Banshee.AudioCd
             get { return disc_model.Duration; }
         }
         
+        public long FileSize {
+            get { return disc_model.FileSize; }
+        }
+        
         public bool DiscIsPlaying {
             get {
                 AudioCdTrackInfo playing_track = ServiceManager.PlayerEngine.CurrentTrack as AudioCdTrackInfo;
@@ -258,6 +262,14 @@ namespace Banshee.AudioCd
         public override int Count {
             get { return disc_model.Count; }
         }
+
+        public override bool HasEditableTrackProperties {
+            get { return true; }
+        }
+
+        public override bool HasViewableTrackProperties {
+            get { return true; }
+        }
         
 #endregion
         
