@@ -29,7 +29,6 @@
 
 using System;
 using System.Collections.Generic;
-using System.Data;
 using System.IO;
 using System.Threading;
 
@@ -57,27 +56,28 @@ namespace Banshee.CoverArt
         private DateTime last_scan = DateTime.MinValue;
         private TimeSpan retry_every = TimeSpan.FromDays (7);
 
-        private static string query = @"
-                SELECT {0}
-                FROM CoreAlbums, CoreArtists, CoreTracks
-                WHERE
-                    CoreAlbums.ArtistID = CoreArtists.ArtistID AND
-                    CoreTracks.AlbumID = CoreAlbums.AlbumID AND
-                    CoreTracks.DateUpdatedStamp > ? AND
-                    CoreTracks.PrimarySourceID = ? AND
-                    CoreTracks.AlbumID NOT IN (
-                        SELECT AlbumID FROM CoverArtDownloads WHERE
-                            LastAttempt > ? OR Downloaded = 1)
-                {1}";
+        private static HyenaSqliteCommand count_query = new HyenaSqliteCommand (@"
+            SELECT count(DISTINCT CoreTracks.AlbumID)
+            FROM CoreTracks
+            WHERE
+                CoreTracks.PrimarySourceID = ? AND
+                CoreTracks.DateUpdatedStamp > ? AND
+                CoreTracks.AlbumID NOT IN (
+                    SELECT AlbumID FROM CoverArtDownloads WHERE
+                        LastAttempt > ? OR Downloaded = 1)");
 
-        private static HyenaSqliteCommand count_query = new HyenaSqliteCommand (String.Format (query,
-            "count(DISTINCT CoreAlbums.AlbumID)", null
-        ));
-
-        private static HyenaSqliteCommand select_query = new HyenaSqliteCommand (String.Format (query,
-            "DISTINCT CoreAlbums.AlbumID, CoreAlbums.Title, CoreArtists.Name",
-            "LIMIT ?"
-        ));
+        private static HyenaSqliteCommand select_query = new HyenaSqliteCommand (@"
+            SELECT DISTINCT CoreAlbums.AlbumID, CoreAlbums.Title, CoreArtists.Name, CoreTracks.Uri, CoreTracks.UriType
+            FROM CoreTracks, CoreArtists, CoreAlbums
+            WHERE
+                CoreTracks.PrimarySourceID = ? AND
+                CoreTracks.DateUpdatedStamp > ? AND
+                CoreTracks.AlbumID = CoreAlbums.AlbumID AND 
+                CoreAlbums.ArtistID = CoreArtists.ArtistID AND
+                CoreTracks.AlbumID NOT IN (
+                    SELECT AlbumID FROM CoverArtDownloads WHERE
+                        LastAttempt > ? OR Downloaded = 1)
+            GROUP BY CoreTracks.AlbumID LIMIT ?");
         
         public CoverArtJob (DateTime lastScan) : base (Catalog.GetString ("Downloading Cover Art"))
         {
@@ -96,14 +96,14 @@ namespace Banshee.CoverArt
         public void Start ()
         {
             Register ();
-            Scheduler.Schedule (this);
+            Scheduler.Schedule (this, JobPriority.Lowest);
         }
 
-        private IDataReader RunQuery ()
+        private HyenaDataReader RunQuery ()
         {
-            return ServiceManager.DbConnection.Query (select_query,
-                last_scan, ServiceManager.SourceManager.MusicLibrary.DbId, last_scan - retry_every, BatchSize
-            );
+            return new HyenaDataReader (ServiceManager.DbConnection.Query (select_query,
+                ServiceManager.SourceManager.MusicLibrary.DbId, last_scan, last_scan - retry_every, BatchSize
+            ));
         }
         
         public void Run ()
@@ -117,21 +117,24 @@ namespace Banshee.CoverArt
             try {
                 DatabaseTrackInfo track = new DatabaseTrackInfo ();
                 while (true) {
-                    total = current + ServiceManager.DbConnection.Query<int> (count_query, last_scan, ServiceManager.SourceManager.MusicLibrary.DbId, last_scan - retry_every);
+                    total = current + ServiceManager.DbConnection.Query<int> (count_query, ServiceManager.SourceManager.MusicLibrary.DbId, last_scan, last_scan - retry_every);
                     if (total == 0 || total <= current) {
                         break;
                     }
 
-                    using (IDataReader reader = RunQuery ()) {
+                    using (HyenaDataReader reader = RunQuery ()) {
                         while (reader.Read ()) {
                             if (IsCancelRequested) {
                                 Finish ();
                                 return;
                             }
                             
-                            track.AlbumTitle = reader.GetString (1);
-                            track.ArtistName = reader.GetString (2);
-                            track.AlbumId = Convert.ToInt32 (reader[0]);
+                            track.AlbumTitle = reader.Get<string> (1);
+                            track.ArtistName = reader.Get<string> (2);
+                            track.PrimarySource = ServiceManager.SourceManager.MusicLibrary;
+                            track.Uri = track.PrimarySource.UriAndTypeToSafeUri (reader.Get<TrackUriType> (4), reader.Get<string> (3));
+                            track.AlbumId = reader.Get<int> (0);
+                            //Console.WriteLine ("have album {0}/{1} for track uri {2}", track.AlbumId, track.AlbumTitle, track.Uri);
 
                             Progress = (double) current / (double) total;
                             Status = String.Format (Catalog.GetString ("{0} - {1}"), track.ArtistName, track.AlbumTitle);
