@@ -27,6 +27,7 @@
 //
 
 using System;
+using System.IO;
 using Mono.Unix;
 using Gtk;
 
@@ -38,6 +39,8 @@ using Banshee.ServiceStack;
 using Banshee.Collection;
 using Banshee.Collection.Database;
 using Banshee.Configuration;
+using Banshee.Kernel;
+using Banshee.Playlist;
 
 using Banshee.Gui;
 
@@ -51,13 +54,10 @@ namespace Banshee.FileSystemQueue
         private bool play_enqueued = false;
         private string path_to_play;
         
-        protected override string TypeUniqueId {
-            get { return "file-system-queue"; }
-        }
-        
         public FileSystemQueueSource () : base (Catalog.GetString ("File System Queue"), 
             Catalog.GetString ("File System Queue"), "file-system-queue", 30)
         {
+            TypeUniqueId = "file-system-queue";
             Properties.SetString ("Icon.Name", "system-file-manager");
             Properties.Set<bool> ("AutoAddSource", false);
             IsLocal = true;
@@ -89,7 +89,7 @@ namespace Banshee.FileSystemQueue
             actions_loaded = true;
             
             UpdateActions ();
-            ServiceManager.SourceManager.ActiveSourceChanged += delegate { UpdateActions (); };
+            ServiceManager.SourceManager.ActiveSourceChanged += delegate { Banshee.Base.ThreadAssist.ProxyToMain (UpdateActions); };
             TrackModel.Reloaded += OnTrackModelReloaded;
             
             Reload ();
@@ -97,12 +97,17 @@ namespace Banshee.FileSystemQueue
             play_enqueued = ApplicationContext.CommandLine.Contains ("play-enqueued");
             
             foreach (string path in ApplicationContext.CommandLine.Files) {
-                Enqueue (path);
+                // If it looks like a URI with a protocol, leave it as is
+                if (System.Text.RegularExpressions.Regex.IsMatch (path, "^\\w+\\:\\/")) {
+                    Log.DebugFormat ("URI file : {0}", path);
+                    Enqueue (path);
+                } else {
+                    Log.DebugFormat ("Relative file : {0} -> {1}", path, Path.GetFullPath (path));
+                    Enqueue (Path.GetFullPath (path));
+                }
             }
         }
         
-        uint source_activate_id = 0;
-        
         public void Enqueue (string path)
         {
             lock (this) {
@@ -124,16 +129,20 @@ namespace Banshee.FileSystemQueue
                             }
                         });
                     };
+
+                    importer.Finished += delegate {
+                        if (visible) {
+                            Banshee.Base.ThreadAssist.ProxyToMain (delegate {
+                                ServiceManager.SourceManager.SetActiveSource (this);
+                            });
+                        }
+                    };
                 }
             
-                importer.Enqueue (path);
-                
-                if (source_activate_id == 0) {
-                    source_activate_id = GLib.Timeout.Add (500, delegate {
-                        ServiceManager.SourceManager.SetActiveSource (this);
-                        source_activate_id = 0;
-                        return false;
-                    });
+                if (PlaylistFileUtil.PathHasPlaylistExtension (path)) {
+                    PlaylistFileUtil.ImportPlaylistToLibrary (path, this, importer);
+                } else {
+                    importer.Enqueue (path);
                 }
             }
         }
