@@ -46,6 +46,7 @@ namespace Banshee.NowPlaying
         public FullscreenWindow (Window parent) : base (WindowType.Toplevel)
         {
             Title = parent.Title;
+            AppPaintable = true;
             
             this.parent = parent;
             this.action_service = ServiceManager.Get<InterfaceActionService> ();
@@ -55,6 +56,12 @@ namespace Banshee.NowPlaying
             SetupWidget ();
         }
         
+        protected override bool OnExposeEvent (Gdk.EventExpose evnt)
+        {
+            evnt.Window.DrawRectangle (Style.BlackGC, true, Allocation);
+            return base.OnExposeEvent (evnt);
+        }
+        
         protected override bool OnKeyPressEvent (Gdk.EventKey evnt)
         {
             PlayerEngineService player = ServiceManager.PlayerEngine;
@@ -145,6 +152,10 @@ namespace Banshee.NowPlaying
         protected override void OnShown ()
         {
             base.OnShown ();
+            if (Child != null) {
+                Child.Show ();
+            }
+            
             OnHideCursorTimeout ();
             ConfigureWindow ();
             HasFocus = true;
@@ -155,7 +166,6 @@ namespace Banshee.NowPlaying
         {
             base.OnHidden ();
             DestroyControls ();
-            parent.RemoveNotification ("is-active", ParentActiveNotification);
         }
         
         private void OnScreenSizeChanged (object o, EventArgs args)
@@ -171,6 +181,7 @@ namespace Banshee.NowPlaying
                 parent.GdkWindow.SkipPagerHint = false;
                 parent.GdkWindow.SkipTaskbarHint = false;
                 Hide ();
+                parent.RemoveNotification ("is-active", ParentActiveNotification);
             } else {
                 parent.GdkWindow.SkipPagerHint = true;
                 parent.GdkWindow.SkipTaskbarHint = true;
