@@ -161,8 +161,8 @@ namespace Banshee.Podcasting.Gui
 
             ServiceManager.SourceManager.ActiveSourceChanged += HandleActiveSourceChanged;
             
-            source.TrackModel.Selection.Changed += delegate { UpdateItemActions (); };
-            source.FeedModel.Selection.Changed += delegate { UpdateFeedActions (); };
+            source.TrackModel.Selection.Changed += delegate { Banshee.Base.ThreadAssist.ProxyToMain (UpdateItemActions); };
+            source.FeedModel.Selection.Changed += delegate { Banshee.Base.ThreadAssist.ProxyToMain (UpdateFeedActions); };
             
             UpdateFeedActions ();
             UpdateItemActions ();
@@ -179,8 +179,10 @@ namespace Banshee.Podcasting.Gui
 
         private void HandleActiveSourceChanged (SourceEventArgs args)
         {
-            UpdateFeedActions ();
-            UpdateItemActions ();
+            Banshee.Base.ThreadAssist.ProxyToMain (delegate {
+                UpdateFeedActions ();
+                UpdateItemActions ();
+            });
         }
 
 #endregion
@@ -457,6 +459,11 @@ namespace Banshee.Podcasting.Gui
                 if (track.Item.IsRead != markRead) {
                     track.Item.IsRead = markRead;
                     track.Item.Save ();
+
+                    if (track.Item.IsRead ^ track.PlayCount > 0) {
+                        track.PlayCount = track.Item.IsRead ? 1 : 0;
+                        track.Save (false);
+                    }
                     any = true;
                 }
             }
